package com.sf.vertx.service.impl;

import java.util.List;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.sf.vertx.api.pojo.AppConfig;
import com.sf.vertx.api.pojo.Node;
import com.sf.vertx.api.pojo.VertxConfig;
import com.sf.vertx.constans.RedisConfig;
import com.sf.vertx.service.AppConfigService;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class AppConfigServiceImpl implements AppConfigService {
	@Value("${server.vertx.environment}")
    private String vertxEnvironment;
	@Autowired
	private RedisTemplate<String, String> redisTemplate;
	
	/***
	 * 从redis加载数据
	 */
	public ConcurrentHashMap<String, AppConfig> loadAllConfig() {
		ConcurrentHashMap<String, AppConfig> cacheAppConfig = new ConcurrentHashMap<>();
		// 获取所有app列表
		Set<String> set = redisTemplate.opsForZSet().range(RedisConfig.APP_CONFIG_SET_KEY, 0, -1);
		for(String appCode : set) {
			String appCodeKey = RedisConfig.APP_CONFIG_PREFIX_KEY + ":" + appCode;
			String appCodeValue = redisTemplate.opsForValue().get(appCodeKey);
			if(StringUtils.isNotBlank(appCodeValue)) {
				AppConfig appConfig = JSON.parseObject(appCodeValue, AppConfig.class);
				cacheAppConfig.put(appCode, appConfig);
				
				//JSONObject jsonObject = JSONObject.parseObject(appCodeValue);//从请求体里获得jsonObject
				//String oldGoodsStorageModes = jsonObject.getString("nodeList");//解析成字符串
				//字符串转list
				//List<Node> oldGoodsStoragemodes = JSON.parseArray(oldGoodsStorageModes,Node.class);
				//log.info("oldGoodsStoragemodes:{}", JSON.toJSONString(oldGoodsStoragemodes));
			}
		}
		log.info("cacheAppConfig:{}", JSON.toJSONString(cacheAppConfig));
		return cacheAppConfig;
	}
	
	public AppConfig getAppConfig(String appCode) {
		String appCodeKey = RedisConfig.APP_CONFIG_PREFIX_KEY + ":" + appCode;
		String appCodeValue = redisTemplate.opsForValue().get(appCodeKey);
		if(StringUtils.isNotBlank(appCodeValue)) {
			AppConfig appConfig = JSONObject.parseObject(appCodeValue, AppConfig.class);
			return appConfig;
		}
		return null;
	}
	
	/***
	 * 新增、修改
	 * @param appConfig
	 */
	public void addAppConfig(AppConfig appConfig) {
		redisTemplate.opsForZSet().add(RedisConfig.APP_CONFIG_SET_KEY, appConfig.getAppCode(), 0);
		String appCodeKey = RedisConfig.APP_CONFIG_PREFIX_KEY + ":" + appConfig.getAppCode();
		redisTemplate.opsForValue().set(appCodeKey, JSONObject.toJSONString(appConfig));
		
		// 发送redis队列,vertx处理
		//String queue = RedisConfig.VETX_ENVIRONMENT_KEY+vertxEnvironment+":list";
		
	}
	
	/***
	 * 删除
	 * @param appConfig
	 */
	public void deleteAppConfig(AppConfig appConfig) {
		redisTemplate.opsForZSet().remove(RedisConfig.APP_CONFIG_SET_KEY, appConfig.getAppCode());
		String appCodeKey = RedisConfig.APP_CONFIG_PREFIX_KEY + ":" + appConfig.getAppCode();
		redisTemplate.delete(appCodeKey);
		
		
		// 发送redis队列,vertx处理
		//String queue = RedisConfig.VETX_ENVIRONMENT_KEY+vertxEnvironment+":list";
	}
	
	
	/***
	 * 加载vertx配置
	 */
	public VertxConfig loadVertxConfig() {
		String vertxConfigKey = RedisConfig.VERTX_CONFIG_STRING_KEY;
		String vertxConfigValue = redisTemplate.opsForValue().get(vertxConfigKey);
		if(StringUtils.isNotBlank(vertxConfigValue)) {
			VertxConfig vertxConfig = JSONObject.parseObject(vertxConfigValue, VertxConfig.class);
			return vertxConfig;
		}
		return null;
	}
	
	/***
	 * 新增、修改
	 * @param appConfig
	 */
	public void addVertxConfig(VertxConfig vertxConfig) {
		String vertxConfigKey = RedisConfig.VERTX_CONFIG_STRING_KEY;
		redisTemplate.opsForValue().set(vertxConfigKey, JSONObject.toJSONString(vertxConfig));
		
		// 发送redis队列,vertx处理
		//String queue = RedisConfig.VETX_ENVIRONMENT_KEY+vertxEnvironment+":list";
	}
}
